﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Articles.DirectXDeviceHandling.Gui;

namespace Articles.DirectXDeviceHandling
{
    public partial class MainWindow : Form
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="MainWindow"/> class.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Creates a new DirectX 11 context targeting the given hardware.
        /// </summary>
        private void CreateDirectXContext(TargetHardware targetHardware)
        {
            m_panContainer.Controls.Clear();
            try
            {
                GraphicsCore.Unload();
                GraphicsCore.Initialize(targetHardware, false);
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    this,
                    ex.ToString(),
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            finally
            {
                m_panGraphics = new Direct3D11Canvas()
                {
                    Dock = DockStyle.Fill
                };
                m_panContainer.Controls.Add(m_panGraphics);
            }

            m_cmdDirectX11.BackColor = Color.WhiteSmoke;
            m_cmdDirectX9.BackColor = Color.WhiteSmoke;
            m_cmdSoftware.BackColor = Color.WhiteSmoke;
        }

        /// <summary>
        /// Called when user wants to open DirectX11 context.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnCmdDirectX11Click(object sender, EventArgs e)
        {
            CreateDirectXContext(TargetHardware.DirectX11);
            m_cmdDirectX11.BackColor = Color.LightBlue;
        }

        /// <summary>
        /// Called when user wants to open DirectX9 context.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnCmdDirectX9Click(object sender, EventArgs e)
        {
            CreateDirectXContext(TargetHardware.DirectX9);
            m_cmdDirectX9.BackColor = Color.LightBlue;
        }

        /// <summary>
        /// Called when user wants to open Software context.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnCmdSoftwareClick(object sender, EventArgs e)
        {
            CreateDirectXContext(TargetHardware.SoftwareRenderer);
            m_cmdSoftware.BackColor = Color.LightBlue;
        }

        /// <summary>
        /// Called when user wants to clear current state.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnCmdClearClick(object sender, EventArgs e)
        {
            m_panContainer.Controls.Clear();
            try
            {
                GraphicsCore.Unload();
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    this,
                    ex.ToString(),
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            finally
            {
                m_panGraphics = new Direct3D11Canvas()
                {
                    Dock = DockStyle.Fill,
                };
                m_panContainer.Controls.Add(m_panGraphics);
            }

            m_cmdDirectX11.BackColor = Color.WhiteSmoke;
            m_cmdDirectX9.BackColor = Color.WhiteSmoke;
            m_cmdSoftware.BackColor = Color.WhiteSmoke;
        }

        /// <summary>
        /// Called when user clicks close.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnCmdCloseClick(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
